--[[
	button.lua
		Scripts for Bongos Action Buttons, which add more features to blizzard action buttons

	Changes over the normal action buttons:
		Colored red when out of range
		Hotkeys are shortened
		
	TODO
		Write a new paging system
		Write a new stance switching system
--]]

local MAX_BUTTONS = 120 --the current maximum amount of action buttons
local id = {}

local function OnUpdate()
	BActionButton.OnUpdate(this, arg1)
end

local function PostClick()
	BActionButton.PostClick(this)
end

local function OnDragStart()
	BActionButton.OnDragStart(this)
end

local function OnReceiveDrag()
	BActionButton.OnReceiveDrag(this)
end

local function OnEnter()
	BActionButton.OnEnter(this)
end

local function OnLeave()
	BActionButton.OnLeave(this)
end

local function OnAttributeChanged()
	if arg1 == 'state-parent' then
		this.stateChanged = true
	end

	if not bg_showGrid then
		BActionButton.HideGrid(this)
	end
end

BActionButton = {
	--Create an Action Button with the given ID and parent
	Create = function(id, parent)
		local name = 'BActionButton' .. id
		local button = CreateFrame('CheckButton', name, parent.driver, 'SecureActionButtonTemplate, ActionButtonTemplate')
		button.flashing = 0
		button.flashtime = 0

		button:SetID(id)
		button:RegisterForDrag("LeftButton", "RightButton")
		button:RegisterForClicks('anyUp')
		button:SetAttribute('type', 'action')
		button:SetAttribute('action', id)
		button:SetAttribute('useparent-statebutton', true)
		button:SetAttribute('useparent-unit', true)
		button:SetAttribute('checkselfcast', true)
		parent.driver:SetAttribute('addchild', button)
--[[
		if id <= 12 then
			button:SetAttribute('*helpbutton-p0', 'heal')
			button:SetAttribute('*action-heal', id + 12)
		end
--]]
		button:SetScript('OnAttributeChanged', OnAttributeChanged)
		button:SetScript('OnUpdate', OnUpdate)

		button:SetScript('PostClick', PostClick)

		button:SetScript('OnDragStart', OnDragStart)
		button:SetScript('OnReceiveDrag', OnReceiveDrag)

		button:SetScript('OnEnter', OnEnter)
		button:SetScript('OnLeave', OnLeave)

		BActionButton.Style(button)
		BActionButton.UpdateAll(button)
		BActionButton.UpdateAllStances(button)
		BActionButton.UpdateAllPages(button)
		
		if not BActionSets_HotkeysShown() then
			getglobal(name .. "HotKey"):Hide()
		end	
		if not BActionSets_MacrosShown() then
			getglobal(name .. "Name"):Hide()
		end

		return button
	end,
	
	Release = function(button)
		button:SetParent(nil)
		button:ClearAllPoints()
		CQ.Hide(button)
		button:SetAttribute('useparent-statebutton', nil)
	end,
	
	Style = function(button)
		local name = button:GetName()
		getglobal(name .. 'Icon'):SetTexCoord(0.06, 0.94, 0.06, 0.94)
		getglobal(name .. 'Border'):SetVertexColor(0, 1, 0, 0.6)
		getglobal(name .. 'NormalTexture'):SetVertexColor(1, 1, 1, 0.5)
	end,
	
	UpdateStance = function(button, stance)
		local barID = button:GetParent():GetParent():GetID()
		local id = button:GetID()
		
		local stanceID = mod(id + BActionBar.GetStanceOffset(barID, stance), MAX_BUTTONS)
		if stanceID == 0 then
			stanceID = 120
		end
		if id == stanceID then
			button:SetAttribute('*action-s' .. stance, nil)
		else
			button:SetAttribute('*action-s' .. stance, stanceID)
		end
	end,
	
	UpdateAllStances = function(button)
		local barID = button:GetParent():GetParent():GetID()
		local stances = BActionBar.GetStances(barID)
		if stances then
			for stance in pairs(stances) do
				BActionButton.UpdateStance(button, stance)
			end
		end
	end,
	
	UpdatePage = function(button, page)
		local barID = button:GetParent():GetParent():GetID()
		local id = button:GetID()

		
		local pagedID = mod(id + BActionBar.GetPageOffset(barID, page), MAX_BUTTONS)
		if pagedID == 0 then
			pagedID = 120
		end
		if BActionBar.CanPage(barID) then
			button:SetAttribute('*action-p' .. page, pagedID)
		else
			button:SetAttribute('*action-p' .. page, id)
		end
	end,
	
	UpdateAllPages = function(button)
		for page = 1, BActionBar.GetNumber() do
			BActionButton.UpdatePage(button, page)
		end
	end,

	--[[ OnX Functions ]]--

	PostClick = function(button)
		if bg_showGrid then
			PickupAction(BActionButton.GetPagedID(button))
		end
		BActionButton.UpdateState(button)
	end,

	OnDragStart = function(button)
		if not BActionSets_ButtonsLocked() or (BActionSets_IsQuickMoveKeyDown() or bg_showGrid) then
			PickupAction(BActionButton.GetPagedID(button))
			BActionButton.UpdateState(button)
		end
	end,

	OnReceiveDrag = function(button)
		PlaceAction(BActionButton.GetPagedID(button))
		BActionButton.UpdateState(button)
	end,

	OnEnter = function(button)
		if BOptionsBinder_SetButton then
			BOptionsBinder_SetButton(button)
		end
		BActionButton.UpdateTooltip(button)
	end,

	OnLeave = function(button)
		button.updateTooltip = nil
		GameTooltip:Hide()
	end,

	--OnUpdate
	OnUpdate = function(button, elapsed)
		local name = button:GetName()
		
		if not getglobal(name .. "Icon"):IsShown() then return end

		--update flashing
		if button.flashing == 1 then
			button.flashtime = button.flashtime - elapsed
			if button.flashtime <= 0 then
				local overtime = -button.flashtime
				if overtime >= ATTACK_BUTTON_FLASH_TIME then
					overtime = 0
				end
				button.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime

				local flashTexture = getglobal(name .. "Flash")
				if flashTexture:IsShown() then
					flashTexture:Hide()
				else
					flashTexture:Show()
				end
			end
		end

		-- Handle range indicator
		if button.rangeTimer then
			if button.rangeTimer < 0 then
				local pagedID = BActionButton.GetPagedID(button)
				local hotkey = getglobal(name .. "HotKey")

				if IsActionInRange(pagedID) == 0 then
					hotkey:SetVertexColor(1, 0.1, 0.1)
	
					if BActionSets_ColorOutOfRange() and IsUsableAction(pagedID) then
						local r, g, b = BActionSets_GetRangeColor()
						getglobal(name .. "Icon"):SetVertexColor(r, g, b)
					end
				else
					hotkey:SetVertexColor(0.6, 0.6, 0.6)

					if IsUsableAction(pagedID) then
						getglobal(name .. "Icon"):SetVertexColor(1, 1, 1)
					end
				end
				button.rangeTimer = TOOLTIP_UPDATE_TIME
			else
				button.rangeTimer = button.rangeTimer - elapsed
			end
		end

		--Tooltip stuff, probably for the cooldown timer
		if button.updateTooltip then
			button.updateTooltip = button.updateTooltip - elapsed
			if button.updateTooltip <= 0 then
				if GameTooltip:IsOwned(button) then
					BActionButton.UpdateTooltip(button)
				else
					button.updateTooltip = nil
				end
			end
		end
	end,

	--[[ Update Functions ]]--

	--Updates the icon, count, cooldown, usability color, if the button is flashing, if the button is equipped,  and macro text.
	Update = function(button)
		local name = button:GetName()
		local action = BActionButton.GetPagedID(button)

		local icon = getglobal(name .. 'Icon')
		local cooldown = getglobal(name .. 'Cooldown')
		local texture = GetActionTexture(action)

		if texture then
			icon:SetTexture(texture)
			icon:Show()
			button.rangeTimer = -1

			button:SetNormalTexture('Interface\\Buttons\\UI-Quickslot2')
		else
			icon:Hide()
			cooldown:Hide()
			button.rangeTimer = nil

			button:SetNormalTexture('Interface\\Buttons\\UI-Quickslot')
			getglobal(name .. 'HotKey'):SetVertexColor(0.6, 0.6, 0.6)
		end

		BActionButton.UpdateCount(button)

		if HasAction(action) then
--			if not button:GetAttribute('statehidden') then
				CQ.Show(button)
--			end

			BActionButton.UpdateState(button)
			BActionButton.UpdateUsable(button)
			BActionButton.UpdateCooldown(button)
			BActionButton.UpdateFlash(button)
		else
			if not(bg_showGrid or BActionSets_ShowGrid()) then
				CQ.Hide(button)
--				button:Hide()
			else
				cooldown:Hide()
			end
		end

		-- Add a green border if button is an equipped item
		local border = getglobal(name .. 'Border')
		if IsEquippedAction(action) then
			border:SetVertexColor(0, 1, 0, 0.6)
			border:Show()
		else
			border:Hide()
		end

		if GameTooltip:IsOwned(button) then
			BActionButton.UpdateTooltip(button)
		else
			button.updateTooltip = nil
		end

		-- Update Macro Text
		getglobal(name .. 'Name'):SetText(GetActionText(action))
	end,

	--Update the cooldown timer
	UpdateCooldown = function(button)
		local start, duration, enable = GetActionCooldown(BActionButton.GetPagedID(button))
		CooldownFrame_SetTimer(getglobal(button:GetName().."Cooldown"), start, duration, enable)
	end,
	
	UpdateCount = function(button)
		local text = getglobal(button:GetName() .. 'Count')
		local action = BActionButton.GetPagedID(button)
	
		if IsConsumableAction(action) then
			text:SetText(GetActionCount(action))
		else
			text:SetText('')
		end
	end,
	
	UpdateHotkey = BBasicActionButton.UpdateHotkey,
	
	StartFlash = function(button)
		button.flashing = 1
		button.flashtime = 0
	
		BActionButton.UpdateState(button)
	end,
	
	StopFlash = function(button)
		button.flashing = 0
	
		getglobal(button:GetName() .. 'Flash'):Hide()
		BActionButton.UpdateState(button)
	end,
	
	UpdateFlash = function(button)
		local action = BActionButton.GetPagedID(button)
		if (IsAttackAction(action) and IsCurrentAction(action)) or IsAutoRepeatAction(action) then
			BActionButton.StartFlash(button)
		else
			BActionButton.StopFlash(button)
		end
	end,

	UpdateTooltip = function(button)
		if BActionSets_TooltipsShown() then
			if GetCVar("UberTooltips") == "1" then
				GameTooltip_SetDefaultAnchor(GameTooltip, button)
			else
				GameTooltip:SetOwner(button, "ANCHOR_RIGHT")
			end

			if GameTooltip:SetAction(BActionButton.GetPagedID(button)) then
				button.updateTooltip = TOOLTIP_UPDATE_TIME
			else
				button.updateTooltip = nil
			end
		end
	end,
	
	--Update if a button is checked or not
	UpdateState = function(button)
		local action = BActionButton.GetPagedID(button)
		button:SetChecked(IsCurrentAction(action) or IsAutoRepeatAction(action))
	end,
	
		--colors the action button if out of mana, out of range, etc
	UpdateUsable = function(button)
		local action = BActionButton.GetPagedID(button)
		local icon = getglobal(button:GetName().."Icon")

		local isUsable, notEnoughMana = IsUsableAction(action)
		if isUsable then
			if BActionSets_ColorOutOfRange() and IsActionInRange(action) == 0 then
				local r, g, b = BActionSets_GetRangeColor()
				icon:SetVertexColor(r, g, b)
			else
				icon:SetVertexColor(1, 1, 1)
			end
		elseif notEnoughMana then
			--Make the icon blue if out of mana
			icon:SetVertexColor(0.5, 0.5, 1)
		else
			--Skill unusable
			icon:SetVertexColor(0.3, 0.3, 0.3)
		end
	end,

	UpdateAll = function(button)
		BActionButton.Update(button)
		BActionButton.UpdateHotkey(button)
	end,

	--[[ Toggles ]]--

	--Show Empty Buttons
	ShowGrid = function(button)
		CQ.Show(button)
	end,

	--Hide Empty Buttons
	HideGrid = function(button)
		if not HasAction(BActionButton.GetPagedID(button)) then
			CQ.Hide(button)
		end
	end,

	ShowMacroText = function(button, enable)
		if enable then
			getglobal(button:GetName().."Name"):Show()
		else
			getglobal(button:GetName().."Name"):Hide()
		end
	end,

	--[[ Action ID Functions ]]--
	
	--Data is cached because requesting the current state of a button is kind of slow
	GetPagedID = function(self)
		if self:GetParent() then
			if self.stateChanged or not id[self] then
				self.stateChanged = nil
				id[self] = SecureButton_GetModifiedAttribute(self, 'action', SecureStateChild_GetEffectiveButton(self)) or 1
			end
			return id[self]
		end
		return 1
	end,
}