--[[
	BActionButtonMain
		This is the event handler for Bongos ActonBars and ActionButtons.

	TODO:
		Nothing, I think.  Should be fairly optimized.
--]]

bg_showGrid = nil --is determines if empty buttons are being shown or not.
local eventHandler

--[[ Helper Functions ]]--

--all action buttons
function BActionMain_ForAllButtons(action, arg1)
	local numActionBars = BActionBar.GetNumber()
	for i = 1, numActionBars do
		if getglobal("BActionBar" .. i) then
			for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
				local button = getglobal("BActionButton" .. j)
				if button and button:GetParent() then
					action(button, arg1)
				else
					break
				end
			end
		end
	end
end

--all visible action buttons
function BActionMain_ForAllShownBars(action, arg1)
	local numActionBars = BActionBar.GetNumber()
	for i = 1, numActionBars do
		local bar = getglobal("BActionBar" .. i)
		if bar and bar:IsShown() then
			for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
				local button = getglobal("BActionButton" .. j)
				if button then
					action(button, arg1)
				else
					break
				end
			end
		end
	end
end

function BActionMain_ForAllShownButtons(action, arg1)
	local numActionBars = BActionBar.GetNumber()
	for i = 1, numActionBars do
		local bar = getglobal("BActionBar" .. i)
		if bar and bar:IsShown() then
			for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
				local button = getglobal("BActionButton" .. j)
				if button then
					if button:IsShown() then
						action(button, arg1)
					end
				else
					break
				end
			end
		end
	end
end

--[[ Update Functions ]]--

local function UpdateButtonsWithID(id)
	local numActionBars = BActionBar.GetNumber()
	if id == 0 then
		BActionMain_ForAllShownBars(BActionButton.Update)
	else
		for i = 1, numActionBars do
			local bar = getglobal("BActionBar" .. i)
			if bar and bar:IsShown() then
				for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
					local button = getglobal("BActionButton" .. j)
					if button and BActionButton.GetPagedID(button) == id then
						BActionButton.Update(button)
					end
				end
			end
		end
	end
end

local function UpdatePageBars()
	local numActionBars = BActionBar.GetNumber()
	
	for i = 1, numActionBars do	
		if BActionBar.CanPage(i) then
			local bar = getglobal("BActionBar" .. i)
			if bar and bar:IsShown() then
				for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
					local button = getglobal("BActionButton" .. j)
					if button then
						BActionButton.Update(button)
					end
				end
			end
		end
	end
end

local function UpdateStanceBars()
	local numActionBars = BActionBar.GetNumber()

	for i = 1, numActionBars do	
		local stances = BActionBar.GetStances(i)
		if stances and next(stances) then
			local bar = getglobal("BActionBar" .. i)
			if bar and bar:IsShown() then
				for j = BActionBar.GetStart(i), BActionBar.GetEnd(i) do
					local button = getglobal("BActionButton" .. j)
					if button then
						BActionButton.Update(button)
					end
				end
			end
		end
	end
end

function BActionMain_UpdatePages()
	for i = 1, BActionBar.GetNumber() do
		if BActionBar.CanPage(i) then
			BActionBar.UpdatePages(i)
		end
	end
end

--[[ Event Handler ]]--

local function OnEvent()
	if event == 'ACTIONBAR_SLOT_CHANGED' then
		UpdateButtonsWithID(arg1)
	elseif event == 'PLAYER_ENTERING_WORLD' then
		BActionMain_ForAllShownBars(BActionButton.Update)
	elseif event == 'UPDATE_STEALTH' or event =='UPDATE_SHAPESHIFT_FORM' then
		UpdateStanceBars()
	elseif event == 'PLAYER_TARGET_CHANGED' then
		BActionMain_ForAllShownBars(BActionButton.Update)
	elseif event == 'ACTIONBAR_PAGE_CHANGED' then
		UpdatePageBars()
	elseif event == 'ACTIONBAR_SHOWGRID' then
		bg_showGrid = true
		BActionMain_ForAllShownBars(BActionButton.ShowGrid)
	elseif event == 'ACTIONBAR_HIDEGRID' then
		bg_showGrid = nil
		BActionMain_ForAllShownBars(BActionButton.HideGrid)
	elseif event == 'UPDATE_BINDINGS' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateHotkey)
	elseif event == 'PLAYER_AURAS_CHANGED' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateUsable)
	elseif event == 'UNIT_INVENTORY_CHANGED' then
		if arg1 == 'player' then
			BActionMain_ForAllShownButtons(BActionButton.Update)
		end
	elseif event == 'ACTIONBAR_UPDATE_STATE' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateState)
	elseif event == 'ACTIONBAR_UPDATE_USABLE' or event == 'UPDATE_INVENTORY_ALERTS' or event == 'ACTIONBAR_UPDATE_COOLDOWN' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateCooldown)
		BActionMain_ForAllShownButtons(BActionButton.UpdateUsable)
	elseif event == 'CRAFT_SHOW' or event == 'CRAFT_CLOSE' or event == 'TRADE_SKILL_SHOW' or event == 'TRADE_SKILL_CLOSE' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateState)
	elseif event == 'PLAYER_ENTER_COMBAT' or event == 'PLAYER_LEAVE_COMBAT' or event == 'START_AUTOREPEAT_SPELL' or event == 'STOP_AUTOREPEAT_SPELL' then
		BActionMain_ForAllShownButtons(BActionButton.UpdateFlash)
	end
end

local function RegisterEvents(frame)
	eventHandler = frame

	frame:SetScript('OnEvent', OnEvent)
	
	frame:RegisterEvent('UPDATE_SHAPESHIFT_FORM')
	frame:RegisterEvent('UPDATE_STEALTH')
	frame:RegisterEvent('ACTIONBAR_PAGE_CHANGED')
	frame:RegisterEvent('PLAYER_TARGET_CHANGED')

	frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	frame:RegisterEvent("ACTIONBAR_SHOWGRID")
	frame:RegisterEvent("ACTIONBAR_HIDEGRID")	
	frame:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
	frame:RegisterEvent("UPDATE_BINDINGS")

	frame:RegisterEvent('ACTIONBAR_UPDATE_STATE')
	frame:RegisterEvent('ACTIONBAR_UPDATE_USABLE')
	frame:RegisterEvent('ACTIONBAR_UPDATE_COOLDOWN')
	frame:RegisterEvent('UPDATE_INVENTORY_ALERTS')
	frame:RegisterEvent('PLAYER_AURAS_CHANGED')
	frame:RegisterEvent('UNIT_INVENTORY_CHANGED')
	frame:RegisterEvent('CRAFT_SHOW')
	frame:RegisterEvent('CRAFT_CLOSE')
	frame:RegisterEvent('TRADE_SKILL_SHOW')
	frame:RegisterEvent('TRADE_SKILL_CLOSE')
	frame:RegisterEvent('PLAYER_ENTER_COMBAT')
	frame:RegisterEvent('PLAYER_LEAVE_COMBAT')
	frame:RegisterEvent('START_AUTOREPEAT_SPELL')
	frame:RegisterEvent('STOP_AUTOREPEAT_SPELL')
end

--[[ Startup ]]--

BProfile.AddStartup(function()
	BState_LoadStateMaps()
	
	for i = 1, BActionBar.GetNumber() do BActionBar.Create(i) end

	if not eventHandler then
		RegisterEvents(CreateFrame("Frame"))
	end
end)