--[[
	Bongos Bindings
		Based off of ClickBinder by Gello and TrinityButtons by Maul/Osk
--]]

--local key, button
local bindFrame

local function Binder_ToBinding(button)
	return 'CLICK ' .. button:GetName() .. ':LeftButton'
end

local function Binder_GetBinding(button)
	return GetBindingKey(Binder_ToBinding(button))
end

local function Binder_UnbindAction(button)
	if not InCombatLockdown() and button then
		local binding = Binder_ToBinding(button)

		while GetBindingKey(binding) do
			SetBinding(GetBindingKey(binding), nil)
		end
		SaveBindings(GetCurrentBindingSet())
	elseif button then
		BMsg("Sorry, you can't change key bindings while in combat.")
	end
end

local function Binder_SetKeybinding(button, key)
	if not InCombatLockdown() then
		if button and key then
			Binder_UnbindAction(button)
	
			SetBindingClick(key, button:GetName(), 'LeftButton')
			SaveBindings(GetCurrentBindingSet())
			BMsg('Set ' .. key .. ' to ' .. button:GetName())
		end
	else
		BMsg("Sorry, you can't change key bindings while in combat.")
	end
end

function Binder_OnKeyDown(binder, newKey)
	local button = binder.button
	
	if newKey == 'UNKNOWN' or newKey == 'SHIFT' or newKey == 'CTRL' or newKey == 'ALT' then return end
	
	local screenshotKey = GetBindingKey('SCREENSHOT')
	if screenshotKey and newKey == screenshotKey then
		Screenshot()
		return
	end
	
	local openChatKey = GetBindingKey('OPENCHAT')
	if openChatKey and newKey == openChatKey then
		ChatFrameEditBox:Show()
		return
	end

	if newKey == "LeftButton" or newKey == "RightButton" then
		return
	elseif newKey == "MiddleButton" then
		newKey = "BUTTON3"
	elseif newKey == "Button4" then
		newKey = "BUTTON4"
	elseif newKey == "Button5" then
		newKey = "BUTTON5"
	end
	
	if newKey == 'ESCAPE' then
		if button then
			Binder_UnbindAction(button)
			BOptionsBinder_SetButton(button)
			BMsg('Cleared Bindings for ' .. button:GetName())
		end
		return
	end

	local key = newKey
	if IsShiftKeyDown() then
		key = 'SHIFT-' .. key
	end
	if IsControlKeyDown() then
		key = 'CTRL-' .. key
	end
	if IsAltKeyDown() then
		key = 'ALT-' .. key
	end
	
	if button then
		local prevAction = GetBindingAction(key)
		if prevAction and prevAction ~= '' and prevAction ~= Binder_ToBinding(button) then
			BMsg('Unbound key from ' .. prevAction)
		end
		Binder_SetKeybinding(button, key)
		BOptionsBinder_SetButton(button)
	end
end

--[[ OnX Functions ]]--

function BOptionsBinder_OnLoad()
	bindFrame = CreateFrame('Button')
	bindFrame.parent = this

	bindFrame:RegisterForClicks('anyUp')
	bindFrame:SetFrameStrata('DIALOG')
	bindFrame:EnableKeyboard(true)
	bindFrame:EnableMouseWheel(true)

	local bg = bindFrame:CreateTexture()
	bg:SetTexture(0, 0, 0, 0.5)
	bg:SetAllPoints(bindFrame)

	local text = bindFrame:CreateFontString('OVERLAY')
	text:SetFontObject('GameFontNormal')
	text:SetAllPoints(bindFrame)
	bindFrame.text = text

	bindFrame:SetScript('OnClick', function() Binder_OnKeyDown(this, arg1) end)
	bindFrame:SetScript('OnKeyDown', function() Binder_OnKeyDown(this, arg1) end)
	bindFrame:SetScript('OnMouseWheel', function()
		if arg1 > 0 then
			Binder_OnKeyDown(this, 'MOUSEWHEELUP')
		else
			Binder_OnKeyDown(this, 'MOUSEWHEELDOWN')
		end
	end)
	bindFrame:SetScript('OnLeave', function() BOptionsBinder_SetButton(nil) end)
	bindFrame:Hide()
end

function BOptionsBinder_OnShow()
	BActionMain_ForAllButtons(BActionButton.ShowGrid)
	BPetBar_ForAllButtons(BPetButton.ShowGrid)
end

function BOptionsBinder_OnHide()
	if not BActionSets_ShowGrid() then
		BActionMain_ForAllButtons(BActionButton.HideGrid)
		BPetBar_ForAllButtons(BPetButton.HideGrid)
	end

	BOptionsBinder_SetButton(nil)
end

--[[ Usable Functions ]]--

function BOptionsBinder_SetButton(button)
	local parentName = bindFrame.parent:GetName()
	
	if button and bindFrame.parent:IsVisible() then
		bindFrame.button = button
		bindFrame.text:SetText(BBasicActionButton.GetHotkey(button))
		bindFrame:SetAllPoints(button)
		bindFrame:Show()
		
		getglobal(parentName .. 'Button'):SetText(button:GetName())
		getglobal(parentName .. 'Binding'):SetText(Binder_GetBinding(button) or 'None')
	elseif bindFrame.parent:IsVisible() then
		bindFrame.button = nil
		bindFrame:ClearAllPoints()
		bindFrame:Hide()
		
		getglobal(parentName .. 'Button'):SetText('None')
		getglobal(parentName .. 'Binding'):SetText('None')
	end
end